EnableExplicit

Global ClientHandle

Structure DOT11_SSID
  uSSIDLength.l
  ucSSID.c[32]
EndStructure

Structure WLAN_ASSOCIATION_ATTRIBUTES
  dot11Ssid.DOT11_SSID
  dot11BssType.q
  dot11Bssid.c[6]
  dot11PhyType.q
  uDot11PhyIndex.l
  wlanSignalQuality.l
  ulRxRate.l
  ulTxRate.l
EndStructure

Structure WLAN_SECURITY_ATTRIBUTES
  bSecurityEnabled.b
  bOneXEnabled.b
  dot11AuthAlgorithm.l
  dot11CipherAlgorithm.l
EndStructure

Structure WLAN_CONNECTION_ATTRIBUTES
  isState.l
  wlanConnectionMode.l
  strProfileName.w[256]
  wlanAssociationAttributes.WLAN_ASSOCIATION_ATTRIBUTES
  wlanSecurityAttributes.WLAN_SECURITY_ATTRIBUTES
EndStructure 

Structure WLAN_INTERFACE_INFO_LIST_NB
  dwNumberOfItems.l
  dwIndex.l
EndStructure

Structure WLAN_INTERFACE_INFO
  InterfaceGuid.GUID
  InterfaceDescription.u[256]
  isState.l
EndStructure

Structure WlanRouter
  WlanInterface.s
  InterfaceGuid.l
  SSID.s
  MAC.s
  Procent.l
  IP.s
  Connected.l
  SaveSetting.s
  Profile.s
EndStructure

Global WlanRouter.WlanRouter

Prototype WlanOpenHandle(dwClientVersion,pReserved,pdwNegotiatedVersion, phClientHandle) 
Prototype WlanCloseHandle(hClientHandle,pReserved)
Prototype WlanEnumInterfaces(hClientHandle,pReserved,ppInterfaceList)
Prototype WlanFreeMemory(pMemory)
Prototype WlanQueryInterface(hClientHandle, pInterfaceGuid, OpCode, pReserved, pdwDataSize, ppData, pWlanOpcodeValueType)
Prototype WlanGetAvailableNetworkList(hClientHandle, pInterfaceGuid, dwFlags, pReserved, ppAvailableNetworkList)

Global WlanOpenHandle.WlanOpenHandle
Global WlanCloseHandle.WlanCloseHandle
Global WlanEnumInterfaces.WlanEnumInterfaces
Global WlanFreeMemory.WlanFreeMemory
Global WlanQueryInterface.WlanQueryInterface
Global WlanGetAvailableNetworkList.WlanGetAvailableNetworkList

Define wI_info.WLAN_INTERFACE_INFO

#wlan_interface_state_not_ready = 0
#wlan_interface_state_connected = 1
#wlan_interface_state_ad_hoc_network_formed = 2
#wlan_interface_state_disconnecting = 3
#wlan_interface_state_disconnected = 4
#wlan_interface_state_associating = 5
#wlan_interface_state_discovering = 6
#wlan_interface_state_authenticating = 7 

Procedure LoadWlanapi(Library) 
  Protected Result 

  Result = OpenLibrary(Library, "wlanapi.dll") 
  
  If Result 
    If Library = #PB_Any 
      Library = Result 
    EndIf 
    
    WlanOpenHandle = GetFunction(Library, "WlanOpenHandle") 
    WlanCloseHandle = GetFunction(Library, "WlanCloseHandle") 
    WlanEnumInterfaces = GetFunction(Library, "WlanEnumInterfaces") 
    WlanFreeMemory = GetFunction(Library, "WlanFreeMemory") 
    WlanQueryInterface = GetFunction(Library, "WlanQueryInterface") 
    WlanGetAvailableNetworkList = GetFunction(Library, "WlanGetAvailableNetworkList")
    
    If  WlanOpenHandle = 0 Or  WlanCloseHandle = 0 Or WlanEnumInterfaces = 0 Or WlanFreeMemory = 0 Or WlanQueryInterface = 0 Or WlanGetAvailableNetworkList = 0
      CloseLibrary(Library) 
      Result = 0 
    EndIf      
  EndIf 

  ProcedureReturn Result 
EndProcedure 

Procedure.s Wlan_Query_interface(pInterfaceGuid)
  Protected lRet.l
  Protected datasize.l=0
  Protected ppdata.l
  Protected opvalue.l
  Protected connection.WLAN_CONNECTION_ATTRIBUTES
  Protected SSID$
  
  lRet = WlanQueryInterface(ClientHandle, pInterfaceGuid, 7, #Null, @datasize,@ppdata,@opvalue.l)
  If lRet <> #ERROR_SUCCESS
    ;Debug "Error function WlanGetAvailableNetworkList :" +Str(lRet)
    ProcedureReturn ""
  EndIf
  
  ;Debug SizeOf(WLAN_CONNECTION_ATTRIBUTEzS)
  
  CopyMemory(ppdata, connection ,SizeOf(WLAN_CONNECTION_ATTRIBUTES))
  
  SSID$ = PeekS(@connection\wlanAssociationAttributes\dot11Ssid\ucSSID[0],-1,#PB_Ascii)
  
  WlanFreeMemory(ppdata)
  
  ProcedureReturn SSID$
EndProcedure  

Procedure.s Enumerate_interface()
  Protected list_interface.WLAN_INTERFACE_INFO_LIST_NB
  Protected pList_interface.l
  Protected lRet.l
  Shared wI_info.WLAN_INTERFACE_INFO
  Protected stGuid.s
  Protected stEtat.s
  Protected Dim wiInterfaces.WLAN_INTERFACE_INFO(0)
  Protected iInterface.l
  Protected connected.l=0
  
  lRet = WlanEnumInterfaces(ClientHandle, #Null, @pList_interface)
  
  If lRet <> #ERROR_SUCCESS
    Debug "Error function WlanEnumInterfaces :" +Str(lRet)
    ProcedureReturn ""
  EndIf
  
  CopyMemory(pList_interface, list_interface,SizeOf(WLAN_INTERFACE_INFO_LIST_NB))
  
  If list_interface\dwNumberOfItems = 0 
    Debug "---------------- No Wifi interface ---------------"
    ProcedureReturn ""
  EndIf
  
  ReDim wiInterfaces(list_interface\dwNumberOfItems - 1)
  
  CopyMemory(pList_interface + SizeOf(WLAN_INTERFACE_INFO_LIST_NB), wiInterfaces(0), SizeOf(WLAN_INTERFACE_INFO) * list_interface\dwNumberOfItems)
  
  For iInterface = 0 To list_interface\dwNumberOfItems - 1
    ;Debug "interface Nr: " +Str(iInterface)
    wI_info = wiInterfaces(iInterface)
    ;Debug "GUID :"+GetGUID(wI_info\InterfaceGuid)
    ;Debug "interface Wifi Description :" + PeekS(@wI_info\InterfaceDescription,-1,#PB_Unicode)
    WlanRouter\WlanInterface=PeekS(@wI_info\InterfaceDescription,-1,#PB_Unicode)
    ;Debug "Interface Wifi State: "+WLAN_INTERFACE_STATE(wI_info\IsState)
    If wI_info\IsState = 1
      WlanRouter\Connected=1
      WlanRouter\InterfaceGuid=wI_info\InterfaceGuid
      connected=1
    Else
      connected=0
    EndIf  
    
  Next
  
  ;ProcedureReturn connected
  ProcedureReturn PeekS(@wI_info\InterfaceDescription,-1,#PB_Unicode)
EndProcedure

Procedure  Terminate_enumeration()
  Protected lRet.l
  lRet = WlanCloseHandle(ClientHandle, #Null)
  
  If lRet <> #ERROR_SUCCESS
    Debug "Error on function WlanCloseHandle :" +Str(lRet)
  EndIf
  
EndProcedure

Procedure.s GetWLANSSID(NetworkName$)
  Protected lRet
  Protected AskedVersion
  Protected NegotiatedVersion
  Protected SSID$
  Protected CurrentNetworkName$
  
  If OSVersion() > #PB_OS_Windows_XP
    AskedVersion=2
  Else
    AskedVersion=1
  EndIf
  
  If LoadWlanapi(0)
    lRet = WlanOpenHandle(AskedVersion, #Null, @NegotiatedVersion, @ClientHandle)
    If lRet <> #ERROR_SUCCESS
      ;Debug "Error function WlanOpenHandle :" +Str(lRet)
      CloseLibrary(0)
      
      ProcedureReturn ""
    EndIf
    
    CurrentNetworkName$ = Enumerate_interface()
    If CurrentNetworkName$ = NetworkName$
      SSID$ = Wlan_Query_interface(WlanRouter\InterfaceGuid)
    EndIf  
    
    ;noch prfen ob guid dieselbe ist wie die von der wlan-karte
    
    Terminate_enumeration()
    
    CloseLibrary(0) 
    
  Else
    Debug "load failed"
  EndIf   
  
  ProcedureReturn SSID$
EndProcedure

;Vorher die Bezeichnung anpassen
MessageRequester("", "WLAN-SSID: " + GetWLANSSID("Realtek PCIe GBE Family Controller"), 0)